/*
** $Header: /u/spert/src/intvec/nv/RCS/mem.c,v 1.6 1995/10/12 01:07:24 davidj Exp $
**
** Naive memory allocation routines
*/


#include <intv-config.h>
#include <assert.h>
#include <stdio.h>
#include <stddef.h>

#ifdef INTV_STDC_HEADERS
#include <stdlib.h>
#else /* not INTV_STDC_HEADERS */
char *malloc();
char *realloc();
void free();
#endif /* not INTV_STDC_HEADERS */
#ifdef INTV_HAVE_MEMALIGN
char* memalign();
#endif

#include <math.h>
#include <intv-defs.h>
#include <intv-scansrc.h>

#ifdef DECL_nv_falloc_vi8
# ifdef INTV_HAVE_ALLOCA
#   define nv_falloc_vi8(len) ((intv_int8_t *) alloca((len) * sizeof(intv_int8_t)))
# else
#   define nv_falloc_vi8(len) \
        ((intv_int8_t *) intv_malloc((len) * sizeof(intv_int8_t)))
# endif
#endif

#ifdef DECL_nv_falloc_vi16
# ifdef INTV_HAVE_ALLOCA
#   define nv_falloc_vi16(len) \
        ((intv_int16_t *) alloca((len) * sizeof(intv_int16_t)))
# else
#   define nv_falloc_vi16(len) \
        ((intv_int16_t *) intv_malloc((len) * sizeof(intv_int16_t)))
# endif
#endif

#ifdef DECL_nv_falloc_vi32
# ifdef INTV_HAVE_ALLOCA
#   define nv_falloc_vi32(len) \
        ((intv_int32_t *) alloca((len) * sizeof(intv_int32_t)))
# else
#   define nv_falloc_vi32(len) \
        ((intv_int32_t *) intv_malloc((len) * sizeof(intv_int32_t)))
# endif
#endif

#ifdef DECL_nv_ffree_vi16
# ifdef INTV_HAVE_ALLOCA
#  define nv_ffree_vi16(ptr)
# else
#  define nv_ffree_vi16(ptr) intv_free((ptr))
# endif
#endif

#ifdef DECL_nv_ffree_vi32
# ifdef INTV_HAVE_ALLOCA
#  define nv_ffree_vi32(ptr)
# else
#  define nv_ffree_vi32(ptr) intv_free((ptr))
# endif
#endif

#ifdef DECL_nv_intv_malloc
void* nv_intv_malloc(size_t size);
#endif

#ifdef DEFN_nv_intv_malloc
void*
nv_intv_malloc(size_t size)
{
    void* ptr;

    ptr = malloc(size);
    if (ptr==NULL)
    {
	fprintf(stderr, "(nv_intv_malloc): ERROR - no free memory\n");
	assert(0);
    }
    return (ptr);
}
#endif

#ifdef DECL_nv_intv_realloc
void* nv_intv_realloc(void* ptr, size_t size);
#endif

#ifdef DEFN_nv_intv_realloc
void*
nv_intv_realloc(void* ptr, size_t size)
{
    void* nptr;

    nptr = realloc(ptr, size);
    if (nptr==NULL)
    {
	fprintf(stderr, "(nv_intv_realloc): ERROR - no free memory\n");
	assert(0);
    }
    return(nptr);
}
#endif

#ifdef DECL_nv_intv_memalign
void* nv_intv_memalign(size_t alignment, size_t size);
#endif

#ifdef DEFN_nv_intv_memalign
void*
nv_intv_memalign(size_t alignment, size_t size)
{
    void* ptr;
#ifdef HAVE_MEMALIGN
    ptr = memalign(alignment, size);
#else
    ptr = malloc(size);
#endif
    if (ptr==NULL)
    {
	fprintf(stderr, "(nv_intv_memalign): ERROR - no free memory\n");
	assert(0);
    }
    return(ptr);
}
#endif


#ifdef DECL_nv_intv_free
void nv_intv_free(void* ptr);
#endif

#ifdef DEFN_nv_intv_free
void
nv_intv_free(void* ptr)
{
    free(ptr);
}
#endif

#ifdef DECL_nv_malloc_vi8
#define nv_malloc_vi8(len) \
    ((intv_int8_t *) intv_malloc((len) * sizeof(intv_int16_t)))
#endif

#ifdef DECL_nv_malloc_vi16
#define nv_malloc_vi16(len) \
    ((intv_int16_t *) intv_malloc((len) * sizeof(intv_int16_t)))
#endif

#ifdef DECL_nv_malloc_vi32
#define nv_malloc_vi32(len) \
    ((intv_int32_t *) intv_malloc((len) * sizeof(intv_int32_t)))
#endif

#ifdef DECL_nv_realloc_vi8
#define nv_realloc_vi8(ptr, len) \
    ((intv_int8_t *) intv_realloc((void *)(ptr), (len)*sizeof(intv_int8_t)))
#endif

#ifdef DECL_nv_realloc_vi16
#define nv_realloc_vi16(ptr, len) \
    ((intv_int16_t *) intv_realloc((void *)(ptr), (len)*sizeof(intv_int16_t)))
#endif

#ifdef DECL_nv_realloc_vi32
#define nv_realloc_vi32(ptr, len) \
    ((intv_int32_t *) intv_realloc((void *) (ptr), (len)*sizeof(intv_int32_t)))
#endif

#ifdef DECL_nv_free_vi8
#define nv_free_vi8(ptr) intv_free((void *) (ptr))
#endif

#ifdef DECL_nv_free_vi16
#define nv_free_vi16(ptr) intv_free((void *) (ptr))
#endif

#ifdef DECL_nv_free_vi32
#define nv_free_vi32(ptr) intv_free((void *) (ptr))
#endif
