/*
** $Header: /u/spert/src/intvec/nv/RCS/scalar.c,v 1.4 1996/01/04 23:07:41 davidj Exp $
**
** Scalar integer operations - the quick and easy way
*/

#include <intv-config.h>
#include <assert.h>
#include <stddef.h>
#include <intv-defs.h>
#include <intv-scansrc.h>

/****************************************************************/

#ifdef DECL_nv_abs_i32_i32
intv_int32_t nv_abs_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_abs_i32_i32
intv_int32_t
nv_abs_i32_i32(intv_int32_t val)
{
    return(abs(val));
}
#endif

/****************************************************************/

#ifdef DECL_nv_satabs_i32_i32
intv_int32_t nv_satabs_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_satabs_i32_i32
intv_int32_t
nv_satabs_i32_i32(intv_int32_t val)
{
    if (val==INTV_INT32_MIN)
	return INTV_INT32_MAX;
    else
	return(abs(val));
}
#endif

/****************************************************************/

#ifdef DECL_nv_neg_i32_i32
intv_int32_t nv_neg_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_neg_i32_i32
intv_int32_t
nv_neg_i32_i32(intv_int32_t val)
{
    return(-val);
}
#endif


/****************************************************************/
/* Saturating negate */

#ifdef DECL_nv_satneg_i32_i32
intv_int32_t nv_satneg_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_satneg_i32_i32
intv_int32_t
nv_satneg_i32_i32(intv_int32_t val)
{
    if (val==INTV_INT32_MIN)
	return INTV_INT32_MAX;
    else
	return(-val);
}
#endif


/****************************************************************/
/* Count leading zeros */

#ifdef DECL_nv_countlz_i32_i32
intv_int32_t nv_countlz_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_countlz_i32_i32
intv_int32_t
nv_countlz_i32_i32(intv_int32_t val)
{
    intv_uint32_t mask;		/* Mask for current bits being checked */
    intv_uint32_t clear;	/* Which top bits are set */
    int topbits;                /* Number of bits being checked */
    int count;                  /* Bits found clear so far */
    int j;                      /* Counter within one word */

    mask = 0xffff0000u;
    topbits = 16;
    count = 0;

    for (j=0; j<4; j++)
    {
        clear = val & mask;
        if (!clear)
        {
            count += topbits;
            val <<= topbits;
        }
        topbits >>= 1;
        mask <<= topbits;
    }
    clear = val & 0xc0000000;
    if (((intv_int32_t) clear) >=0)
        count++;
    if (clear==0)
        count++;
    return((intv_int32_t) count);
}
#endif

/****************************************************************/
/* Count leading ones */

#ifdef DECL_nv_countlo_i32_i32
intv_int32_t nv_countlo_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_countlo_i32_i32
intv_int32_t
nv_countlo_i32_i32(intv_int32_t val)
{
    return(nv_countlz_i32_i32(~val));
}
#endif

/****************************************************************/
/* Count leading sign bits */

#ifdef DECL_nv_countls_i32_i32
intv_int32_t nv_countls_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_countls_i32_i32
intv_int32_t
nv_countls_i32_i32(intv_int32_t val)
{
    if (val<0)
        return(countlz_i32_i32(~val));
    else
        return(countlz_i32_i32(val));
}
#endif

/****************************************************************/
/* 16 bit swap bytes (little/big endian change) */

#ifdef DECL_nv_swapb_i16_i16
intv_int16_t nv_swapb_i16_i16(intv_int16_t val);
#endif

#ifdef DEFN_nv_swapb_i16_i16
intv_int16_t
nv_swapb_i16_i16(intv_int16_t val)
{
    intv_uint16_t uval;
    intv_uint16_t res;
    intv_int16_t b0, b1;

    uval = (intv_uint16_t) val;
    b0 = uval >> 8;
    b1 = uval << 8;

    res = b0 | b1;

    return (intv_int16_t) res;
}
#endif

/****************************************************************/
/* 32 bit swap bytes (little/big endian change) */

#ifdef DECL_nv_swapb_i32_i32
intv_int32_t nv_swapb_i32_i32(intv_int32_t val);
#endif

#ifdef DEFN_nv_swapb_i32_i32
intv_int32_t
nv_swapb_i32_i32(intv_int32_t val)
{
    intv_uint32_t uval;
    intv_uint32_t res;
    intv_int32_t b0, b1, b2, b3;

    uval = (intv_uint32_t) val;
    b0 = uval >> 24;
    b1 = (uval >> 8) & 0x0000ff00;
    b2 = (uval << 8) & 0x00ff0000;
    b3 = uval << 24;

    res = b0 | b1 | b2 | b3;

    return (intv_int32_t) res;
}
#endif

/****************************************************************/
/* Saturating add */

#ifdef DECL_nv_satadd_i32i32_i32
intv_int32_t nv_satadd_i32i32_i32(intv_int32_t a, intv_int32_t b);
#endif

#ifdef DEFN_nv_satadd_i32i32_i32
intv_int32_t
nv_satadd_i32i32_i32(intv_int32_t a, intv_int32_t b)
{
    intv_int32_t res;		/* The result */
    intv_bool_t ovf;		/* The overflow flag */

    res = a + b;
    /* Find out if the addition overflowed */
    /* ovf = (sign_bit(a)==sign_bit(b) && sign_bit(a) != sign_bit(res)) */
    ovf = ((a^b) >= 0) & ((a^res) < 0);
    if (ovf)
    {
	/* If overflowed, we can use the sign of either of the inputs to
	   decide result */
	if (a>0)
	    res = INTV_INT32_MAX;
	else
	    res = INTV_INT32_MIN;
    }
    return res;
}
#endif

/****************************************************************/
/* Saturating subtract */

#ifdef DECL_nv_satsub_i32i32_i32
intv_int32_t nv_satsub_i32i32_i32(intv_int32_t a, intv_int32_t b);
#endif

#ifdef DEFN_nv_satsub_i32i32_i32
intv_int32_t
nv_satsub_i32i32_i32(intv_int32_t a, intv_int32_t b)
{
    intv_int32_t res;		/* The result */
    intv_bool_t ovf;		/* The overflow flag */

    res = a - b;
    /* Find out if the subtraction overflowed */
    /* ovf = (sign_bit(a)!=sign_bit(b) && sign_bit(a) != sign_bit(diff) */
    ovf = ((a^b) < 0) & ((a^res) < 0);
    if (ovf)
    {
	/* If overflowed, we can use the sign of a to decide result */
	if (a>=0)
	    res = INTV_INT32_MAX;
	else
	    res = INTV_INT32_MIN;
    }
    return res;
}
#endif

/****************************************************************/
/* 32 bit truncating arithmetic shift right */

#ifdef DECL_nv_srt_i32i32_i32
intv_int32_t nv_srt_i32i32_i32(intv_int32_t a, int rshift);
#endif

#ifdef DEFN_nv_srt_i32i32_i32
intv_int32_t
nv_srt_i32i32_i32(intv_int32_t val, int rshift)
{
    assert(((unsigned) rshift)<32);

    return val>>rshift;
}
#endif

/****************************************************************/
/* 16 bit truncating arithmetic shift right */

#ifdef DECL_nv_srt_i16i32_i16
intv_int16_t nv_srt_i16i32_i16(intv_int16_t a, intv_int32_t rshift);
#endif

#ifdef DEFN_nv_srt_i16i32_i16
intv_int16_t
nv_srt_i16i32_i16(intv_int16_t val, intv_int32_t rshift)
{
    assert(((unsigned) rshift)<16);

    return val>>rshift;
}
#endif


/****************************************************************/
/* 32 bit rounding arithmetic shift right */

#ifdef DECL_nv_srr_i32i32_i32
intv_int32_t nv_srr_i32i32_i32(intv_int32_t val, int rshift);
#endif

#ifdef DEFN_nv_srr_i32i32_i32
intv_int32_t
nv_srr_i32i32_i32(intv_int32_t val, int rshift)
{
    intv_int32_t res;		/* Result */
    intv_uint32_t rem;		/* Remainder - bits shifted off right */

    assert(((unsigned) rshift)<32);

    if (rshift==0)
	res = val;
    else
    {
	res = val >> rshift;	/* Rely on arithmetic right shift */
	rem = val << (32-rshift);
	res += rem >> (32-1);	/* If rem has MSB set, add 1 to res */
	if (rem==0x80000000)
	    res &= 0xfffffffe;
    }
    return(res);
}
#endif

/****************************************************************/
/* 16 bit rounding arithmetic shift right */

#ifdef DECL_nv_srr_i16i32_i16
intv_int16_t nv_srr_i16i32_i16(intv_int16_t val, intv_int32_t rshift);
#endif

#ifdef DEFN_nv_srr_i16i32_i16
intv_int16_t
nv_srr_i16i32_i16(intv_int16_t val, intv_int32_t rshift)
{
    intv_int16_t res;		/* Result */
    intv_uint16_t rem;		/* Remainder - bits shifted off right */

    assert(((unsigned) rshift)<16);

    if (rshift==0)
	res = val;
    else
    {
	res = val >> rshift;	/* Rely on arithmetic right shift */
	rem = val << (16-rshift);
	res += rem >> (16-1);	/* If rem has MSB set, add 1 to res */
	if (rem==0x8000)
	    res &= 0xfffe;
    }
    return(res);
}
#endif

/****************************************************************/
/* 32 bit logical shift left */

#ifdef DECL_nv_sll_i32i32_i32
intv_int32_t nv_sll_i32i32_i32(intv_int32_t val, int lshift);
#endif

#ifdef DEFN_nv_sll_i32i32_i32
intv_int32_t
nv_sll_i32i32_i32(intv_int32_t val, int lshift)
{
    assert(((unsigned) lshift) < 32);

    return val << lshift;
}
#endif

/****************************************************************/
/* 16 bit logical shift left */

#ifdef DECL_nv_sll_i16i32_i16
intv_int16_t nv_sll_i16i32_i16(intv_int16_t val, intv_int32_t lshift);
#endif

#ifdef DEFN_nv_sll_i16i32_i16
intv_int16_t
nv_sll_i16i32_i16(intv_int16_t val, intv_int32_t lshift)
{
    assert(((unsigned) lshift) < 16);

    return val << lshift;
}
#endif

/****************************************************************/
/* 32 bit saturating shift left */

#ifdef DECL_nv_sls_i32i32_i32
intv_int32_t nv_sls_i32i32_i32(intv_int32_t val, int lshift);
#endif

#ifdef DEFN_nv_sls_i32i32_i32
intv_int32_t
nv_sls_i32i32_i32(intv_int32_t val, int lshift)
{
    int sign;
    intv_uint32_t mask;
    intv_int32_t res;

    assert(((unsigned) lshift) < 32);
    sign = (val < 0);
    mask = 0xffffffff << (32 - lshift - 1);
    if (!sign)
    {
        if (((intv_uint32_t) val & mask) != 0)
            res = INTV_INT32_MAX;
        else
            res = val << lshift;
    }
    else
    {
        if (((intv_uint32_t) val & mask) != mask)
            res = INTV_INT32_MIN;
        else
            res = val << lshift;
    }
    return(res);
}
#endif


/****************************************************************/
/* 16 bit saturating shift left */

#ifdef DECL_nv_sls_i16i32_i16
intv_int16_t nv_sls_i16i32_i16(intv_int16_t val, intv_int32_t lshift);
#endif

#ifdef DEFN_nv_sls_i16i32_i16
intv_int16_t
nv_sls_i16i32_i16(intv_int16_t val, intv_int32_t lshift)
{
    intv_int32_t temp;		/* 32 bit result */
    intv_int16_t res;

    assert(((unsigned) lshift) < 16);
    temp = (intv_int32_t) val << lshift;
    if (temp>INTV_INT16_MAX)
	res = INTV_INT16_MAX;
    else if (temp<INTV_INT16_MIN)
	res = INTV_INT16_MIN;
    else
	res = (intv_int16_t) temp;

    return res;
}
#endif


