/*
   $Header: /u/spert/src/intvec/testsuite/RCS/copy_i_svi_test.c,v 1.1 1996/10/23 03:10:00 davidj Exp $

   Test element wise vector routines of the form op_vi16_vi32.
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"

typedef void (*rtn8_t)(size_t len, size_t stride, const intv_int8_t in,
		       intv_int8_t* res);

typedef void (*rtn16_t)(size_t len, size_t stride, const intv_int16_t in,
			intv_int16_t* res);

typedef void (*rtn32_t)(size_t len, size_t stride, const intv_int32_t in,
			intv_int32_t* res);

static maxlen;			/* Maximum length of vector. */

/* Performs one test given input values and expected output. */

void
onetest_i8(rtn8_t rtn, size_t len, size_t stride, intv_int8_t in)
{
    size_t i;
    const size_t ext = len * stride;
    const intv_int8_t nin = ~in;

    /* Create new padded result vector. */
    intv_int8_t* c_res = rtst_padvec_new_vi8(len*stride);
    for (i=0; i<ext; i++)
	c_res[i] = nin;

    /* Run routine. */
    rtn(len, stride, in, c_res);

    /* Check result. */
    for (i=0; i<ext; i++)
    {
	if ((i % stride)==0)
	    rtst_assert(in==c_res[i]);
	else
	    rtst_assert(nin==c_res[i]);
    }

    /* Free and check padding. */
    rtst_padvec_del_vi8(c_res);
}

void
onetest_i16(rtn16_t rtn, size_t len, size_t stride, intv_int16_t in)
{
    size_t i;
    const size_t ext = len * stride;
    const intv_int16_t nin = ~in;

    /* Create new padded result vector. */
    intv_int16_t* c_res = rtst_padvec_new_vi16(len*stride);
    for (i=0; i<ext; i++)
	c_res[i] = nin;

    /* Run routine. */
    rtn(len, stride, in, c_res);

    /* Check result. */
    for (i=0; i<ext; i++)
    {
	if ((i % stride)==0)
	    rtst_assert(in==c_res[i]);
	else
	    rtst_assert(nin==c_res[i]);
    }

    /* Free and check padding. */
    rtst_padvec_del_vi16(c_res);
}

void
onetest_i32(rtn32_t rtn, size_t len, size_t stride, intv_int32_t in)
{
    size_t i;
    const size_t ext = len * stride;
    const intv_int32_t nin = ~in;

    /* Create new padded result vector. */
    intv_int32_t* c_res = rtst_padvec_new_vi32(len*stride);
    for (i=0; i<ext; i++)
	c_res[i] = nin;

    /* Run routine. */
    rtn(len, stride, in, c_res);

    /* Check result. */
    for (i=0; i<ext; i++)
    {
	if ((i % stride)==0)
	    rtst_assert(in==c_res[i]);
	else
	    rtst_assert(nin==c_res[i]);
    }

    /* Free and check padding. */
    rtst_padvec_del_vi32(c_res);
}

void
randomtest_i8(rtn8_t rtn)
{
    const int numtests = rtst_numtests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len;
        size_t stride;
	intv_int8_t in;

	len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	stride = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        in = rtst_urand_i8i8_i8(-0x80, 0x7f);
        rtst_log("len=%d stride=%d\n", (int) len, (int) stride);
        onetest_i8(rtn, len, stride, in);
    }
}

void
randomtest_i16(rtn16_t rtn)
{
    const int numtests = rtst_numtests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len;
        size_t stride;
	intv_int16_t in;

	len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	stride = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        in = rtst_urand_i16i16_i16(-0x8000, 0x7fff);
        rtst_log("len=%d stride=%d\n", (int) len, (int) stride);
        onetest_i16(rtn, len, stride, in);
    }
}

void
randomtest_i32(rtn32_t rtn)
{
    const int numtests = rtst_numtests;

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len;
        size_t stride;
	intv_int32_t in;

	len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
	stride = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        in = rtst_urand_i32i32_i32(-0x7fffffff-1, 0x7fffffff);
        rtst_log("len=%d stride=%d\n", (int) len, (int) stride);
        onetest_i32(rtn, len, stride, in);
    }
}


void
test_i8()
{
    rtst_start("copy_i8_svi8");
    rtst_log("testing nv.\n");
    onetest_i8(nv_copy_i8_svi8, 1, 1, 0);
    onetest_i8(nv_copy_i8_svi8, 3, 1, 1);
    onetest_i8(nv_copy_i8_svi8, 10, 2, -13);
    randomtest_i8(nv_copy_i8_svi8);

    rtst_passed();
}

void
test_i16()
{
    rtst_start("copy_i16_svi16");
    rtst_log("testing nv.\n");
    onetest_i16(nv_copy_i16_svi16, 1, 1, 0);
    onetest_i16(nv_copy_i16_svi16, 3, 1, 1);
    onetest_i16(nv_copy_i16_svi16, 10, 2, -13);
    randomtest_i16(nv_copy_i16_svi16);

    rtst_passed();
}

void
test_i32()
{
    rtst_start("copy_i32_svi32");
    rtst_log("testing nv.\n");
    onetest_i32(nv_copy_i32_svi32, 1, 1, 0);
    onetest_i32(nv_copy_i32_svi32, 3, 1, 1);
    onetest_i32(nv_copy_i32_svi32, 10, 2, -13);
    randomtest_i32(nv_copy_i32_svi32);

    rtst_passed();
}


int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    /* Because we are using strided vectors, set maxlen smaller. */
    maxlen = rtst_sizetests / 2;

    test_i8();
    test_i16();
    test_i32();

    rtst_exit();

    return 0;
}
