/*
   $Header: /u/spert/src/intvec/testsuite/RCS/scalar_i32_i32_test.c,v 1.3 1996/01/04 23:26:59 davidj Exp $

   Test scalar 32 bit unary ops
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"

typedef intv_int32_t (*rtn_i32_i32_t)(intv_int32_t in);

void
multitest(rtn_i32_i32_t rtn, size_t len, const intv_int32_t* inv,
	  const intv_int32_t* corv)
{
    size_t i;
    intv_int32_t in;
    intv_int32_t res;
    intv_int32_t correct;

    for (i=0; i<len; i++)
    {
	in = *inv++;
	correct = *corv++;
	res = rtn(in);
	if (res!=correct)
	{
	    rtst_failed("in=%i res=%i correct=%i", in, res, correct);
	}
    }
}

void
test_abs()
{
    const intv_int32_t in[] =
    {
	0, -1, 1, 0x7fff, 0x7fffffff, -0x7fff, -0x8000, -0x7fffffff,
	INTV_INT32_MIN
    };
    const intv_int32_t res[] =
    {
	0, 1, 1, 0x7fff, 0x7fffffff, 0x7fff, 0x8000, 0x7fffffff, INTV_INT32_MIN
    };
    rtst_start("abs_i32_i32");
    multitest(abs_i32_i32, sizeof(in)/sizeof(in[0]), in, res);
    rtst_passed();
}

void
test_neg()
{
    const intv_int32_t in[] =
    {
	0, -1, 1, 0x7fff, 0x7fffffff, -0x7fff, -0x8000, -0x7fffffff,
	INTV_INT32_MIN
    };
    const intv_int32_t res[] =
    {
	0, 1, -1, -0x7fff, -0x7fffffff, 0x7fff, 0x8000, 0x7fffffff, INTV_INT32_MIN
    };
    rtst_start("neg_i32_i32");
    multitest(neg_i32_i32, sizeof(in)/sizeof(in[0]), in, res);
    rtst_passed();
}

void
test_satabs()
{
    const intv_int32_t in[] =
    {
	0, -1, 1, 0x7fff, 0x7fffffff, -0x7fff, -0x8000, -0x7fffffff,
	INTV_INT32_MIN
    };
    const intv_int32_t res[] =
    {
	0, 1, 1, 0x7fff, 0x7fffffff, 0x7fff, 0x8000, 0x7fffffff, INTV_INT32_MAX
    };
    rtst_start("satabs_i32_i32");
    multitest(satabs_i32_i32, sizeof(in)/sizeof(in[0]), in, res);
    rtst_passed();
}

void
test_satneg()
{
    const intv_int32_t in[] =
    {
	0, -1, 1, 0x7fff, 0x7fffffff, -0x7fff, -0x8000, -0x7fffffff,
	INTV_INT32_MIN
    };
    const intv_int32_t res[] =
    {
	0, 1, -1, -0x7fff, -0x7fffffff, 0x7fff, 0x8000, 0x7fffffff, INTV_INT32_MAX
    };
    rtst_start("satneg_i32_i32");
    multitest(satneg_i32_i32, sizeof(in)/sizeof(in[0]), in, res);
    rtst_passed();
}


void
test_swapb()
{
    const intv_int32_t in[] =
    {
	0, 0x12345678, 0xfedcba98
    };
    const intv_int32_t res[] =
    {
	0, 0x78563412, 0x98badcfe
    };
    rtst_start("swapb_i32_i32");
    multitest(swapb_i32_i32, sizeof(in)/sizeof(in[0]), in, res);
    rtst_passed();
}



int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_abs();
    test_satabs();
    test_neg();
    test_satneg();
    test_swapb();
    rtst_exit();

    return 0;
}
