/*
   $Header: /u/spert/src/intvec/testsuite/RCS/vecele_vi16_vi32_test.c,v 1.1 1995/10/10 19:23:59 davidj Exp $

   Test element wise vector routines of the form op_vi16_vi32.
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"


typedef void (*rtn_t)(size_t len, const intv_int16_t* in,
		      intv_int32_t* res);

/* Performs one test given input values and expected output. */

void
onetest_normal(rtn_t rtn, size_t len, const intv_int16_t* in,
	       const intv_int32_t* correct)
{
    /* Create padded copies of inputs. */
    intv_int16_t* c_in = rtst_padvec_clone_vi16(len, in);
    intv_int32_t* c_res = rtst_padvec_new_vi32(len);

    /* Run routine. */
    rtn(len, c_in, c_res);

    /* Check result. */
    rtst_checkeq_vi32vi32(len, c_res, correct);

    /* Check input not corrupted. */
    rtst_checkeq_vi16vi16(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vi16(c_in);
    rtst_padvec_del_vi32(c_res);
}

void
conv1(rtn_t rtn)
{
    const intv_int16_t in[] = { 0, -1, 1, -12345, 0x7fff, -32768};
    const intv_int32_t correct[] = { 0, -1, 1, -12345, 0x7fff, -32768};
    
    onetest_normal(rtn, 6, in, correct);
}

/* Compare the results of two vector routines */

void
random_test(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    intv_int16_t *in = rtst_padvec_new_vi16(maxlen);
    intv_int32_t *res = rtst_padvec_new_vi32(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_badrand_vi16(len, in);
        nv_rtn(len, in, res);

        rtst_log("len=%d\n", len);

        onetest_normal(test_rtn, len, in, res);
    }

    rtst_padvec_del_vi16(in);
    rtst_padvec_del_vi32(res);
}

void
test_conv(void)
{
    rtst_start("conv_vi16_vi32");
    rtst_log("testing nv.\n");
    conv1(nv_conv_vi16_vi32);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    conv1(tor_conv_vi16_vi32);
    random_test(nv_conv_vi16_vi32, tor_conv_vi16_vi32);
#endif
    rtst_passed();
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_conv();

    rtst_exit();

    return 0;
}
